<?php
/**
 * Reporting scope for Max Media Manager
 *
 * Describes whether a report should be for a specific publisher,
 * a specific advertiser or 'everyone'.
 *
 * @since 0.3.19 - Feb 20, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id: ReportScope.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 */

class ReportScope
{
    /* @var int */
    var $_publisher_id;

    /* @var int */
    var $_advertiser_id;

    /* @var string */
    var $description;

    /**
     * PHP4 constructor
     */
    function ReportScope()
    {
        $this->useAllAvailableData();
    }

    function getPublisherId()
    {
        return $this->_publisher_id;
    }

    function getAdvertiserId()
    {
        return $this->_advertiser_id;
    }

    function useAllAvailableData()
    {
        $this->_publisher_id = false;
        $this->_advertiser_id = false;
        $this->description = 'all available advertisers and publishers';
    }

    function usePublisherId($id)
    {
        $this->_publisher_id = $id;
        $this->_advertiser_id = false;
        $this->description = "publisher $id";
    }

    function useAdvertiserId($id)
    {
        $this->_advertiser_id = $id;
        $this->_publisher_id = false;
        $this->description = "advertiser $id";
    }

    function hasNoRestrictions()
    {
        if ($this->_publisher_id) {
            return false;
        }
        if ($this->_advertiser_id) {
            return false;
        }
        return true;
    }

    function useValuesFromQueryArray($values, $base_key)
    {
        $specifier_key = $base_key . '_entity';
        $publisher_key = $base_key . '_publisher';
        $advertiser_key = $base_key . '_advertiser';

        $scope_specifier = $values[$specifier_key];

        if ($scope_specifier == 'publisher') {
            $publisher_id = $values[$publisher_key];
            $this->usePublisherId($publisher_id);
        } elseif ($scope_specifier == 'advertiser') {
            $advertiser_id = $values[$advertiser_key];
            $this->useAdvertiserId($advertiser_id);
        } elseif ($scope_specifier == 'all') {
            $this->useAllAvailableData();
        }else {
            trigger_error("Max was asked to limit scope for something to an enity that it didn't recognize.");
        }
    }
}
?>
