<?php
/**
 * Report scope tests for Max Media Manager
 *
 * @since 0.3.19 - Feb 20, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id: ReportScope.admin.test.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 */

require_once MAX_PATH . '/lib/max/Admin/Reporting/ReportScope.php';

class ReportScopeTest extends UnitTestCase
{
    function testDefaultHasNoRestrictions()
    {
        $scope = new ReportScope();
        $this->assertTrue($scope->hasNoRestrictions());
    }

    function testUsingPublisherEntailsRestrictions()
    {
        $scope = new ReportScope();
        $scope->usePublisherId(316);
        $this->assertFalse($scope->hasNoRestrictions());
    }

    function testUsingAdvertiserEntailsRestrictions()
    {
        $scope = new ReportScope();
        $scope->useAdvertiserId(420);
        $this->assertFalse($scope->hasNoRestrictions());
    }

    function testDescriptionDefault()
    {
        $scope = new ReportScope();
        $this->assertEqual($scope->description, "all available advertisers and publishers");
    }

    function testDescriptionOnPublisher()
    {
        $scope = new ReportScope();
        $scope->usePublisherId(316);
        $this->assertEqual($scope->description, "publisher 316");
    }

    function testDescriptionOnAdvertiser()
    {
        $scope = new ReportScope();
        $scope->useAdvertiserId(420);
        $this->assertEqual($scope->description, "advertiser 420");
    }

    function testUseAdvertiserIsSane()
    {
        $scope = new ReportScope();
        $scope->useAdvertiserId(420);
        $this->assertEqual($scope->getAdvertiserId(), 420);
    }

    function testAdvertisersAreBlankAfterSettingPublisher()
    {
        $scope = new ReportScope();
        $scope->useAdvertiserId(420);
        $scope->usePublisherId(316);
        $this->assertFalse($scope->getAdvertiserId(), 'Using a publisher should mean that no advertisers are selected');
    }

    function testPublishersAreBlankAfterSettingAdvertiser()
    {
        $scope = new ReportScope();
        $scope->usePublisherId(316);
        $scope->useAdvertiserId(420);
        $this->assertFalse($scope->getPublisherId(), 'Using an advertiser should mean that no publishers are selected');
    }

    function testPublishersAreBlankAfterSettingAll()
    {
        $scope = new ReportScope();
        $scope->useAllAvailableData();
        $this->assertFalse($scope->getPublisherId(), 'Using all available data should mean that no publishers are selected');
    }

    function testAdvertisersAreBlankAfterSettingAll()
    {
        $scope = new ReportScope();
        $scope->useAllAvailableData();
        $this->assertFalse($scope->getAdvertiserId(), 'Using all available data should mean that no advertisers are selected');
    }
}
?>
