<?php
/**
 * Zone scope test for Max Media Manager
 *
 * @since 0.3.19 - Feb 24, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id: ZoneScope.admin.test.php 4307 2006-02-24 16:45:08Z roh@m3.net $
 */

require_once MAX_PATH . '/lib/max/Admin/Reporting/ZoneScope.php';

class ZoneScopeTest extends UnitTestCase
{
    function testDefaultIsAllZones()
    {
        $scope = new ZoneScope();
        $this->assertFalse($scope->isSpecificZone());
    }

    function testUseSpecificZoneIsSane()
    {
        $scope = new ZoneScope();
        $scope->useZoneId(321);
        $this->assertEqual($scope->getZoneId(), 321);
    }

    function testSpecificZoneIsSpecificZone()
    {
        $scope = new ZoneScope();
        $scope->useZoneId(321);
        $this->assertTrue($scope->isSpecificZone());
    }

    function testGetZoneIdRaisesErrorOnAllZones()
    {
        $scope = new ZoneScope();
        $scope->useAllZones();
        $zone = $scope->getZoneId();
        $this->assertError(false, 'The zone ID is meaningless when the scope is "all zones", so asking for it should raise an error');
    }

    function testSwitchingBetweenZones()
    {
        $scope = new ZoneScope();
        $scope->useZoneId(321);
        $scope->useZoneId(123);

        $this->assertEqual($scope->getZoneId(), 123);
    }

    function testSwitchingBetweenSpecificZoneAndAll()
    {
        $scope = new ZoneScope();
        $scope->useZoneId(321);
        $scope->useAllZones();

        $this->assertFalse($scope->isSpecificZone());
    }
}
?>
