<?php
/**
 * A period of time between two dates
 *
 * @since 0.3.19 - Feb 13, 2006
 * @copyright 2006 M3 Media Services
 * @version $Id:DateRange.php 4300 2006-02-23 16:38:40Z roh@m3.net $
 */

class DateRange
{
    /* The date/time considered to be 'now'
     *  @var Date */
    var $_now;

    /* @var Date */
    var $_start;

    /* @var Date */
    var $_end;

    /**
     * PHP5 constructor
     *
     * @param Date now
     */
    function __construct($now)
    {
        $this->_now = $now;
        $this->_start = $now;
        $this->_end = $now;
    }
    /**
     * PHP4 constructor
     *
     * @param Date now
     */
    function DateRange($now)
    {
        $this->__construct($now);
    }

    /**
     * Factory method to produce a date range loaded with today's date.
     */
    function newToday($now)
    {
        $range = new DateRange($now);
        $range->useToday();
        return $range;
    }

    function useToday()
    {
        $today_start = $this->_midnight($this->_now);
        $today_end = $this->_add24Hours($today_start);

        $this->_start = $today_start;
        $this->_end = $today_end;
    }

    function useThisWeek()
    {
        $base_date = $this->_now;
        $days_since_sunday = $base_date->getDayOfWeek();
        $sunday = $this->_midnight($base_date);
        $sunday = $this->_subtractDays($sunday, $days_since_sunday);
        $next_sunday = $this->_addDays($sunday, 7);

        $this->_start = $sunday;
        $this->_end = $next_sunday;
    }

    /**
     * Set the date range to the previous Sunday-Sunday period.
     */
    function useLastWeek()
    {
        $base_date = $this->_now;
        $days_since_sunday = $base_date->getDayOfWeek();
        $sunday = $this->_midnight($base_date);
        $sunday = $this->_subtractDays($sunday, $days_since_sunday);
        $previous_sunday = $this->_subtractDays($sunday, 7);

        $this->_start = $previous_sunday;
        $this->_end = $sunday;
    }

    /**
     * Set the date range to be the 7-day period ending yesterday.
     */
    function useLast7Days()
    {
        $week_end = $this->_midnight($this->_now);
        $week_start = $this->_subtractDays($week_end, 7);

        $this->_start = $week_start;
        $this->_end = $week_end;
    }

    /**
     * Set the date range to be from the first of last month to the last.
     */
    function useLastMonth()
    {
        $this_month_start = $this->_startOfMonth($this->_now);
        $last_month_start = $this->_startOfPreviousMonth($this_month_start);

        $this->_start = $last_month_start;
        $this->_end = $this_month_start;
    }

    function useThisMonth()
    {
        $this_month_start = $this->_startOfMonth($this->_now);
        $next_month_sometime = $this->_addDays($this_month_start, 40);
        $next_month_start = $this->_startOfMonth($next_month_sometime);

        $this->_start = $this_month_start;
        $this->_end = $next_month_start;
    }

    function useTextSpecifier($specifier)
    {
        switch ($specifier) {
            case 'lastmonth':
                $this->useLastMonth();
                break;
            case 'lastweek':
                $this->useLastWeek();
                break;
            case 'last7days':
                $this->useLast7Days();
                break;
            case 'thisweek':
                $this->useThisWeek();
                break;
            case 'thismonth':
                $this->useThisMonth();
                break;
            default:
                trigger_error("Max encountered date range description that it didn't recognise: '$specifier'");
                break;
        }
    }

    function _startOfPreviousMonth($this_month_start)
    {
        $day = $this_month_start->getDay();
        $last_month_final_day = $this->_subtractDays($this_month_start, $day);
        $days_last_month = $last_month_final_day->getDay();

        $last_month_start = $this->_subtractDays($this_month_start, $days_last_month);

        return $last_month_start;
    }

    function _midnight($date)
    {
        $processed_date = $date;
        $processed_date->setHour(0);
        $processed_date->setMinute(0);
        $processed_date->setSecond(0);
        return $processed_date;
    }

    function _startOfMonth($date)
    {
        $processed_date = $this->_midnight($date);
        $processed_date->setDay(1);
        return $processed_date;
    }

    function _add24Hours($base_date)
    {
        $modified_date = $base_date;
        $modified_date->addSeconds(60*60*24);
        return $modified_date;
    }

    function _subtractDays($base_date, $days)
    {
        $modified_date = $base_date;
        $span = new Date_Span((string) $days, '%D');
        $modified_date->subtractSpan($span);
        return $modified_date;
    }

    function _addDays($base_date, $days)
    {
        $modified_date = $base_date;
        $span = new Date_Span((string) $days, '%D');
        $modified_date->addSpan($span);
        return $modified_date;
    }

    /**
     * The end of this range.
     *
     * @return Date A date object representing the start of the range
     */
    function getStartDate()
    {
        return $this->_start;
    }

    /**
     * The end of this range.
     *
     * @return Date A date object representing the end of the range
     */
    function getEndDate()
    {
        return $this->_end;
    }

    /**
     * Use the values from a $_GET-style array.
     *
     * @todo Extract some of the logic into separate methods
     */
    function useValuesFromQueryArray($values, $base_key)
    {
        $preset_key = $base_key . '_preset';
        $start_key = $base_key . '_start';
        $end_key = $base_key . '_end';

        $preset_string = $values[$preset_key];
        if (!isset($preset_string)) {
            $preset_string = 'specific';
        }
        $start_string = $values[$start_key];
        $end_string = $values[$end_key];


        if ($preset_string == 'specific') {
            if (!($start_string && $end_string)) {
                trigger_error("Max was asked to generate a date range but wasn't given dates.");
            }
            $this->setDateRangeByStrings($start_string, $end_string);
        } else {
            $this->useTextSpecifier($preset_string);
        }
    }

    /**
     * Set the start and end dates of this range from string representations of dates.
     */
    function setDateRangeByStrings($start_string, $end_string)
    {
        $start_date = new Date($start_string);
        $end_date = new Date($end_string);
        $this->_start = $start_date;
        $this->_end = $end_date;
    }

    function getStartSql()
    {
        $sql = $this->_start->format('%Y-%m-%d %H:%M:%S');
        return "'" . $sql . "'";
    }

    function getEndSqlForComparingTimestamps()
    {
        $sql = $this->_end->format('%Y-%m-%d %H:%M:%S');
        return "'" . $sql . "'";
    }

    function getEndSqlForComparingDays()
    {
        $previous_day = $this->_subtractDays($this->_end, 1);
        $sql = $previous_day->format('%Y-%m-%d');
        return "'" . $sql . "'";
    }

    /**
     * Format the date representing the start of this range.
     *
     * @return string
     */
    function getStartDateForDisplay()
    {
        $display = $this->formatDateForDisplay($this->_start);
        return $display;
    }

    /**
     * Format the date representing the end of this range.
     *
     * More complicated than simply formatting the end date object directly,
     * because the the internal representation of a date-time is quite different
     * from a human concept of the end of a day.
     *
     * @return string
     */
    function getEndDateForDisplay()
    {
        $previous_day = $this->_subtractDays($this->_end, 1);
        $display = $this->formatDateForDisplay($previous_day);
        return $display;
    }

    function formatDateForDisplay($date)
    {
        return $date->format('%d/%m/%Y');
    }
}

?>