<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: ck.php 4146 2006-01-31 15:14:34Z chris@m3.net $
*/

/**
 * @package    MaxDelivery
 * @author     Scott Switzer <scott@switzer.org>
 */

// Require the initialisation file
require_once '../../init-delivery.php';

// Required files
require_once MAX_PATH . '/lib/max/Delivery.php';
require_once MAX_PATH . '/lib/max/other/lib-log.inc.php';

// Get the remote host info
require_once MAX_PATH . '/lib/max/other/lib-remotehost.inc.php';

// Prevent click from being cached by browsers
require_once MAX_PATH . '/lib/max/other/lib-cache.inc.php';
    
// Send P3P headers...
sendP3PHeaders();

// Convert specially encoded params into the $_REQUEST variable
convertMaxParams();

// Remove any special characters
removeSpecialChars($_REQUEST);

// Get the variables
$viewerId   = isset($_COOKIE[$conf['var']['viewerId']]) ? $_COOKIE[$conf['var']['viewerId']] : generateUniqueId();
$adId       = isset($_REQUEST[$conf['var']['adId']]) ? intval($_REQUEST[$conf['var']['adId']]) : 0;
$zoneId     = isset($_REQUEST[$conf['var']['zoneId']]) ? intval($_REQUEST[$conf['var']['zoneId']]) : 0;
$creativeId = isset($_REQUEST[$conf['var']['creativeId']]) ? intval($_REQUEST[$conf['var']['creativeId']]) : 0;

if ($adId == 0 && $zoneId > 0) {
    $adId = _getZoneAd($zoneId);
}

if (($adId > 0) && ($conf['logging']['adClicks']) && !(isset($_GET['log']) && ($_GET['log'] == 'no'))) {
    logAdClick($viewerId, $adId, $creativeId, $zoneId);
}

// Set the userid cookie
setcookie($conf['var']['viewerId'], $viewerId, time() + $conf['cookie']['permCookieSeconds'], '/', (!empty($conf['cookie']['domain']) ? $conf['cookie']['domain'] : null));
    
// Get the URL that we are going to redirect to
$destination = _getDestinationUrl($adId);

// Redirect to the destination url
Header ("Location: $destination");

// Stop benchmarking
stopBenchmark();

/**
 * Populate variables from a specially encoded string.  This is used because
 * in a click URL, a parameter could possibly be another URL.
 */
function convertMaxParams()
{
    $conf = $GLOBALS['_MAX']['CONF'];
    $qs = $_SERVER['QUERY_STRING'];
    // 1.  Strip off the destination
    $dest = false;
    $destStr = $conf['var']['dest'] . '=';
    $pos = strpos($qs, $destStr);
    if ($pos === false) {
        $destStr = 'dest=';
        $pos = strpos($qs, $destStr);
    }
    if ($pos !== false) {
        $dest = urldecode(substr($qs, $pos + strlen($destStr)));
        $qs = substr($qs, 0, $pos);
    }
    // 2.  Parse the remaining string
    $aGet = array();
    $paramStr = $conf['var']['params'] . '=';
    $paramPos = strpos($qs, $paramStr);
    if (is_numeric($paramPos)) {
        $qs = urldecode(substr($qs, $paramPos + strlen($paramStr)));
        $delim = $qs{0};
        if (is_numeric($delim)) {
            $delim = substr($qs, 1, $delim);
        }
        $qs = substr($qs, strlen($delim) + 1);
        MAX_parse_str($qs, $aGet, $delim);
        
        // Fix the destination URL since if appended by a form, it will have no '?'
        $qPos = isset($aGet[$conf['var']['dest']]) ? strpos($aGet[$conf['var']['dest']], '?') : false;
        $aPos = isset($aGet[$conf['var']['dest']]) ? strpos($aGet[$conf['var']['dest']], '&') : false;
        if ($aPos && !$qPos) {
            $desturl = substr($aGet[$conf['var']['dest']], 0, $aPos);
            $destparams = substr($aGet[$conf['var']['dest']], $aPos+1);
            $aGet[$conf['var']['dest']] = $desturl . '?' . $destparams;
        }
    } else {
        parse_str($qs, $aGet);
    }
    if ($dest !== false) {
        $aGet[$conf['var']['dest']] = $dest;
    }
    // 3.  Add any cookie values to the GET string...
    $n = isset($_GET[$conf['var']['n']]) ? $_GET[$conf['var']['n']] : '';
    if (!empty($n) && !empty($_COOKIE[$conf['var']['vars']][$n])) {
        $aVars = unserialize(stripslashes($_COOKIE[$conf['var']['vars']][$n]));
        foreach ($aVars as $name => $value) {
            if (!isset($_GET[$name])) {
                $aGet[$name] = $value;
            }
        }
    }
    $_GET = $aGet;
    $_REQUEST = $_GET + $_POST + $_COOKIE;
}

/**
 * Get the URL that this click will redirect to.  This method first checks
 * to see if the URL is passed in.  If not, it checks the banner from the cache
 * or database.  As a last resort, it either uses the default banner URL or the referer.
 *
 * @param integer $adId The ID of the advertisement that was clicked
 * @return string The destination URL
 */
function _getDestinationUrl($adId)
{
    $conf = $GLOBALS['_MAX']['CONF'];
    $dest = isset($_REQUEST[$conf['var']['dest']]) ? $_REQUEST[$conf['var']['dest']] : '';
    if (empty($dest)) {
        // Include the DB library
        include_once MAX_PATH . '/lib/max/other/db_proc.php';
        // Get the destination from the banner
        $aAd = MAX_getCachedEntity('ad', $adId);
        if (!empty($aAd)) {
            $dest = $aAd['url'];
        }
    }
    /**
     * @TODO Remove code below, as the default banner target needs to go into the
     * cached entity retrieval above!
     */
    //if (empty($dest)) {
    //    $dest = ($adId == 'DEFAULT') ? $pref['default_banner_destination'] : $_SERVER['HTTP_REFERER'];
    //}
    $aVariables = array();
    $aValidVariables = array(
        $conf['var']['adId'],
        $conf['var']['cacheBuster'],
        $conf['var']['channel'],
        $conf['var']['dest'],
        $conf['var']['logClick'],
        $conf['var']['n'],
        $conf['var']['zoneId'],
        $conf['var']['params']
    );
    
    // We also need to ensure that any variables already present in the dest are not duplicated...
    $destParams = parse_url($dest);
    if (!empty($destParams['query'])) {
        $destQuery = explode('&', $destParams['query']);
        if (!empty($destQuery)) {
            foreach ($destQuery as $destPair) {
                list($destName, $destValue) = explode('=', $destPair);
                $aValidVariables[] = $destName;
            }
        }
    }
    
    foreach ($_GET as $name => $value) {
        if (!in_array($name, $aValidVariables)) {
            $aVariables[] = $name . '=' . $value;
        }
    }
    foreach ($_POST as $name => $value) {
        if (!in_array($name, $aValidVariables)) {
            $aVariables[] = $name . '=' . $value;
        }
    }
    if (!empty($aVariables)) {
        $dest .= ((strpos($dest, '?') > 0) ? '&' : '?') . implode('&', $aVariables);
    }
    return $dest;
}

/**
 * Get the ad information when only passed in a zone ID (for email zones)
 *
 * @param int $zoneId The Zone ID of the zone
 * @return int $adId The ad ID of the only linked banner, or 0 if <> 1 active ad linked
 */

function _getZoneAd($zoneId)
{
    $conf = $GLOBALS['conf'];
    
    require_once MAX_PATH . '/lib/max/other/delivery.php';
    $zoneLinkedAds = MAX_getZoneLinkedAds($zoneId, false, true);
    
    if (!empty($zoneLinkedAds['xAds']) && count($zoneLinkedAds['xAds']) == 1) {
        $ad = $zoneLinkedAds['xAds'];
    } elseif (!empty($zoneLinkedAds['ads']) && count($zoneLinkedAds['ads']) == 1) {
        list($adId, $ad) = each($zoneLinkedAds['ads']);
    } elseif (!empty($zoneLinkedAds['lAds']) && count($zoneLinkedAds['lAds']) == 1) {
        list($adId, $ad) = each($zoneLinkedAds['lAds']);
    }        
    
    if (!empty($ad['url'])) {
        // Store the destination URL to save querying the DB again
        $_REQUEST[$conf['var']['dest']] = $ad['url'];
    }
    return $adId;
}

/**
 * Mimic PHP's parse_str functionality (including decoding values), only allow passing in a delimiter.
 *
 * @param string $qs The string in which to parse
 * @param string $aArr The array in which to store the values
 * @param string $delim The delimiter on which to parse the value
 */
function MAX_parse_str($qs, &$aArr, $delim = '&')
{
    $aArr = $_GET;
    // Parse the rest of the array and add to the request array.
    $aElements = explode($delim, $qs);
    foreach($aElements as $element) {
        $len = strpos($element, '=');
        if ($len !== false) {
            $name = substr($element, 0, $len);
            $value = substr($element, $len+1);
            $aArr[$name] = urldecode($value);
        }
    }
}

?>
